﻿using System;
using System.Collections.ObjectModel;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace ThinkGeoSaveShapeDbfColumnsProblem
{
    public partial class Form1 : Form
    {
        private const string InputShape = @"../../../SampleShape/BlockExportTest.shp";
        private const string OutputShape = @"../../../SampleShape/OutputTest.shp";

        private ManagedProj4Projection Proj4 = null;

        public Form1()
        {
            InitializeComponent();
            Proj4 = GetUTMProjection("15", "north");
        }

        public static ManagedProj4Projection GetUTMProjection(string utmZone, string utmHemi)
        {
            ManagedProj4Projection proj4 = new ManagedProj4Projection();
            proj4.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4.ExternalProjectionParametersString = string.Format("+proj=utm +zone={0} +{1} +ellps=WGS84 +datum=WGS84 +units=m +no_defs", utmZone, utmHemi);//UTM

            return proj4;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // Get the base shape
            //ShapeFileFeatureLayer shape = new ShapeFileFeatureLayer(InputShape, ShapeFileReadWriteMode.ReadOnly);

            //TODO: modify the path of file geodatabase.
            var shape = new FileGeoDatabaseFeatureLayer(@"\\TestGDBForThinkGeo.gdb", "\\Blocks");

            shape.FeatureSource.Open();
            var cs = shape.FeatureSource.GetColumns();
            var fs = shape.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);

            shape.Open();
            Collection<Feature> shapeFeatures = shape.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            Collection<FeatureSourceColumn> allColumns = shape.FeatureSource.GetColumns();
            shape.Close();

            // Get the filtered imfl based on the features in the input shape
            InMemoryFeatureLayer filteredLayer = GetIMFL(shapeFeatures, allColumns, shape.ZoomLevelSet, Proj4);

            // Now try and save that shape
            ShapeFiles.SaveShapeFile(filteredLayer, OutputShape, Proj4);

            // Now, open the two shapefiles in ArcGIS or other and look at the columns.
            // You will see that the values and the columns are all jumbled up and some values are altogether missing.
            // If you switch Dependency dir to ThinkGeoDesktopEdition_v7 and use v7 libraries, you will not have any problems.
        }

        private static InMemoryFeatureLayer GetIMFL(Collection<Feature> features, Collection<FeatureSourceColumn> columns, ZoomLevelSet zoom,
            ManagedProj4Projection proj4)
        {
            // Create a new imfl from the base imfl

            InMemoryFeatureLayer imfl = new InMemoryFeatureLayer();
            Collection<string> overLengthColumns = new Collection<string>();

            imfl.Open();
            foreach (FeatureSourceColumn column in columns)
            {
                if (column.ColumnName.Length > 10)
                {
                    overLengthColumns.Add(column.ColumnName);
                    column.ColumnName = column.ColumnName.Substring(0, 10);
                }
                // Updated as per Troy's suggestion.
                imfl.Columns.Add(new FeatureSourceColumn(column.ColumnName, column.TypeName, column.MaxLength));
            }

            int count = 0;
            proj4.Open();

            foreach (Feature feature in features)
            {
                // Normally, I would do some kind of processing here to filter out
                // what I am actually looking for in the original dataset
                // just get every other one to start with
                count++;
                if (count % 2 == 0) continue;
                Feature convertedFeature = proj4.ConvertToExternalProjection(feature);

                foreach (var c in overLengthColumns)
                {
                    convertedFeature.ColumnValues[c.Substring(0, 10)] = convertedFeature.ColumnValues[c];
                }

                imfl.InternalFeatures.Add(convertedFeature);
            }
            proj4.Close();

            imfl.Close();

            return imfl;
        }
    }
}