﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace ThinkGeoSaveShapeDbfColumnsProblem
{
    internal class ShapeFiles
    {
        public static void SaveShapeFile(FeatureLayer fl, string shapeName, ManagedProj4Projection proj4)
        {
            // Get the columns
            Collection<DbfColumn> allColumns = new Collection<DbfColumn>();

            fl.Open();

            foreach (FeatureSourceColumn column in fl.FeatureSource.GetColumns())
            {
                if (column.ColumnName == "Shape" || column.ColumnName == "SHAPE" || column.ColumnName == "OBJECTID")
                    continue;

                DbfColumnType colType = (DbfColumnType)Enum.Parse(typeof(DbfColumnType), column.TypeName);

                int decimalLength = 0;

                if (column.ColumnName.Length > 10)
                {
                    //fl.FeatureSource.Open();
                    //fl.FeatureSource.BeginTransaction();

                    fl.FeatureSource.UpdateColumn(column.ColumnName, new FeatureSourceColumn(column.ColumnName.Substring(0, 10)));

                    fl.FeatureSource.CommitTransaction();

                    column.ColumnName = column.ColumnName.Substring(0, 10);
                }

                allColumns.Add(new DbfColumn(column.ColumnName, colType, column.MaxLength, decimalLength));
            }


            // Get they type of the shape file and output it.  I.e. it may be a Polygon or Points or Area.
            WellKnownType shapeType = fl.FeatureSource.GetFirstFeaturesWellKnownType();
            fl.Close();
            ShapeFileType shapeFileType = new ShapeFileType();

            switch (shapeType)
            {
                case WellKnownType.Polygon:
                    {
                        shapeFileType = ShapeFileType.Polygon;
                        break;
                    }
                case WellKnownType.Line:
                    {
                        shapeFileType = ShapeFileType.Polyline;
                        break;
                    }
                case WellKnownType.Multiline:
                    {
                        shapeFileType = ShapeFileType.Polyline;
                        break;
                    }
                case WellKnownType.Multipoint:
                    {
                        shapeFileType = ShapeFileType.Multipoint;
                        break;
                    }
                case WellKnownType.Point:
                    {
                        shapeFileType = ShapeFileType.Point;
                        break;
                    }
                case WellKnownType.Multipolygon:
                    {
                        shapeFileType = ShapeFileType.Polygon;
                        break;
                    }
            }

            ShapeFileFeatureLayer.CreateShapeFile(shapeFileType,
                shapeName, allColumns, Encoding.ASCII, OverwriteMode.Overwrite);

            ShapeFileFeatureLayer shapeFileLayer = new ShapeFileFeatureLayer(
                shapeName, ShapeFileReadWriteMode.ReadWrite);

            fl.Open();
            proj4.Open();

            foreach (Feature feature in fl.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns))
            {
                shapeFileLayer.Open();
                try
                {
                    Feature convertedFeature = proj4.ConvertToInternalProjection(feature);

                    shapeFileLayer.EditTools.BeginTransaction();
                    shapeFileLayer.EditTools.Add(convertedFeature);
                    TransactionResult result = shapeFileLayer.EditTools.CommitTransaction();
                }
                finally
                {
                    shapeFileLayer.Close();
                }
            }
            fl.Close();

            // Output a prj file as well.
            string prj = ManagedProj4Projection.ConvertProj4ToPrj(proj4.InternalProjectionParametersString);
            string fileName = shapeName.Substring(0, shapeName.Length - 4) + ".prj";
            string folderName = new FileInfo(shapeName).DirectoryName;

            StreamWriter streamFile = new StreamWriter(fileName, false);
            streamFile.WriteLine(prj);
            streamFile.Close();

            string outMessage = string.Format("Output {0}", new FileInfo(shapeName).Name);

            MessageBox.Show(outMessage);
        }
    }
}